#!/usr/bin/env python3

import argparse
import os
import subprocess
import sys

from gettext import gettext as _


KEYDIR = 'server_key'


class CopyKey(object):
    def __init__(self):
        self._parse_args()

    def _parse_args(self):
        self.parser = argparse.ArgumentParser(
            description=_(
                "Copy the server keys to a local USB key"
                )
            )
        self.parser.add_argument(
            '-s', '--server',
            default="proxy",
            help=_(
                "The server on which we want to connect to"
                ),
            )
        self.parser.add_argument(
            'key',
            help=_(
                "Path to the mounted USB key"
                ),
            )
        self.parser.add_argument(
            '-v', '--verbose',
            action='store_true',
            )
        self.parser.add_argument(
            '-f', '--force',
            action='store_true',
            help=_(
                "Overwrite existing files"
                ),
            )
        self.args = self.parser.parse_args()

    def main(self):
        # We need to write our keys on the root directory of some
        # storage.
        if not os.path.ismount(self.args.key):
            if self.args.verbose:
                print(
                    _(
                        "This script requires the root directory where " +
                        "an USB key has been mounted"
                        ),
                    file=sys.stderr,
                    )
            self.parser.error(_(
                "{key} is not a mountpoint".format(key=self.args.key)
                ))

        # Then we need a directory to write the keys into
        keydir_path = os.path.join(self.args.key, KEYDIR)
        try:
            os.makedirs(keydir_path, exist_ok=True)
        except Exception as e:
            self.parser.error(_(
                "Could not create directory {dir}: {e}".format(
                    dir=keydir_path,
                    e=e,
                    )
                ))

        # But we don't want to overwrite existing keys
        if not self.args.force and (
                    os.path.exists(os.path.join(keydir_path, 'client-rsa')) or
                    os.path.exists(os.path.join(keydir_path, 'client-rsa.pub'))
                    ):
                self.parser.error(_(
                    "A key already exists on this key: use --force " +
                    "to overwrite it"
                    ))

        # And finally, we try to copy stuff from the server
        res = subprocess.call([
            'scp',
            '{server}:/etc/fuss-server/client_keys/client-rsa*'.format(
                server=self.args.server,
                ),
            keydir_path,
            ])
        if res == 0 and self.args.verbose:
            print(_(
                (
                    "The keys have been successfully copied to {key}, " +
                    "feel free to umount it and use it to connect " +
                    "clients to the server"
                    ).format(
                        key=self.args.key
                        )
                ))


if __name__ == '__main__':
    CopyKey().main()
